using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace MuoviPallina
{
    public class PallinaGame : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Texture2D pallina, bg;
        Vector2 posizionePallina = new Vector2(0);
        int speed = 1;
        int size = 1;

        public PallinaGame()
        {
            this.Components.Add(new UpdateAndFramePS(this));

            // Permettiamo il ridimensionamento della finestra
            this.Window.AllowUserResizing = true;

            // Inizializziamo il GraphicsDeviceManager e impostiamo le dimensioni
            // desiderate dell'area di disegno
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 800;
            graphics.PreferredBackBufferHeight = 600;

            // Impostiamo la sottocartella con le risorse
            this.Content.RootDirectory = "Content";
        }

        // Carichiamo le risorse in memoria
        protected override void LoadContent()
        {
            // Inizializziamo lo SpriteBatch
            spriteBatch = new SpriteBatch(this.GraphicsDevice);

            // Carichiamo sfondo e personaggio
            bg = this.Content.Load<Texture2D>("sfondo");
            pallina = this.Content.Load<Texture2D>("pallina");

        }


        // Gestiamo la logica di gioco
        protected override void Update(GameTime gameTime)
        {
            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Escape))
                this.Exit();

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Delete))
            {
                speed = 1;
                size = 1;
                posizionePallina.X = 0;
                posizionePallina.Y = 0;
            }

            if (speed >= 1 && Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.PageDown))
                speed--;
            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.PageUp))
                speed++;

            Rectangle gameArea = new Rectangle(0, 0, graphics.GraphicsDevice.Viewport.Width,
                graphics.GraphicsDevice.Viewport.Height);

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.F11) && size >= 1)
                size--;
            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.F12) &&
                gameArea.Contains(this.GetPallinaRectangle(0, 0, 1, 1)))
                size++;


            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Up))
                if (gameArea.Contains(this.GetPallinaRectangle(0, -speed, 0, 0)))
                    posizionePallina.Y -= speed;
                else
                    posizionePallina.Y = 0;

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Down))
                if (gameArea.Contains(this.GetPallinaRectangle(0, +speed, 0, 0)))
                    posizionePallina.Y += speed;
                else
                    posizionePallina.Y = graphics.GraphicsDevice.Viewport.Height - size * pallina.Height;

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Left))
                if (gameArea.Contains(this.GetPallinaRectangle(-speed, 0, 0, 0)))
                    posizionePallina.X -= speed;
                else
                    posizionePallina.X = 0;

            if (Keyboard.GetState(PlayerIndex.One).IsKeyDown(Keys.Right))
                if (gameArea.Contains(this.GetPallinaRectangle(+speed, 0, 0, 0)))
                    posizionePallina.X += speed;
                else
                    posizionePallina.X = graphics.GraphicsDevice.Viewport.Width - size * pallina.Width;



            base.Update(gameTime);
        }

        private Rectangle GetPallinaRectangle()
        {
            return GetPallinaRectangle(0, 0, 0, 0);
        }
        private Rectangle GetPallinaRectangle(int dx, int dy, int dw, int dh)
        {
            return new Rectangle((int)posizionePallina.X + dx, (int)posizionePallina.Y + dy,
                pallina.Width * (size + dw), pallina.Height * (size + dh));
        }

        // Gestiamo l'output grafico istante per istante
        protected override void Draw(GameTime gameTime)
        {
            // Puliamo l'area di disegno
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();

            // Disegnamo lo sfondo
            spriteBatch.Draw(bg, new Rectangle(0, 0, this.GraphicsDevice.Viewport.Width,
                this.GraphicsDevice.Viewport.Height), Color.White);
            // Disegnamo il personaggio
            spriteBatch.Draw(pallina, this.GetPallinaRectangle(), Color.White);

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
